#!/bin/sh -e

prog="`pwd`/pgoblin ${1}"
TF=`mktemp -t pgoblin` || exit 1
${prog} -D 'DROP TABLE pgoblin_test_table' || true

echo 'CREATE TABLE pgoblin_test_table'
cat << EOF > ${TF}
#!${prog}
#perform
CREATE TABLE pgoblin_test_table
     ( t text
     , i int
     )
EOF
chmod a+x ${TF}
${TF}

echo 'COPY in    :test	0&1'
cat << EOF > ${TF}
#!${prog}
#copyin 
COPY pgoblin_test_table FROM stdin
EOF
chmod a+x ${TF}
${TF} << EOF
test	0
test	1
EOF

echo 'COPY in with \ :test	2'
(echo 'test	2'; echo -n '\') | ${prog} -C 'COPY pgoblin_test_table FROM stdin'

echo 'COPY in with \. :test	3'
(echo 'test	3'; echo -n '\.') | ${prog} -C 'COPY pgoblin_test_table FROM stdin'

echo 'COPY in with \.\n :test	4'
${prog} -BC 'COPY pgoblin_test_table FROM stdin' << EOF
test	4
\.
EOF

echo 'COPY in    :test	5'
echo -n 'test	5' | ${prog} -C 'COPY pgoblin_test_table FROM stdin'

echo 'COPY in with \n :test	6'
${prog} -C 'COPY pgoblin_test_table FROM stdin' << EOF
test	6
EOF

echo 'COPY in with \ :test	7'
(echo 'test	7' ; echo -n '\') | ${prog} -C 'COPY pgoblin_test_table FROM stdin'

echo 'COPY in with \. :test	8'
(echo 'test	8' ; echo -n '\.') | ${prog} -C 'COPY pgoblin_test_table FROM stdin'

echo 'COPY in with \.\n :test	9'
${prog} -C 'COPY pgoblin_test_table FROM stdin' << EOF
test	9
\.
EOF

echo 'diff with SELECT'
3<< EOF
|test|0|
|test|1|
|test|2|
|test|3|
|test|4|
|test|5|
|test|6|
|test|7|
|test|8|
|test|9|
EOF
${prog} -s"SELECT '|', t, '|', i, '|
' FROM pgoblin_test_table" | diff /dev/fd/3 /dev/fd/0

echo 'diff with SELECT'
4<< EOF
|test|0|
|test|1|
|test|2|
|test|3|
|test|4|
|test|5|
|test|6|
|test|7|
|test|8|
|test|9|
EOF
${prog} << EOF | diff /dev/fd/4 /dev/fd/0
#strselect 
SELECT '|', t, '|', i, '|
' FROM pgoblin_test_table
EOF

echo 'diff with COPY out'
5<< EOF
test	0
test	1
test	2
test	3
test	4
test	5
test	6
test	7
test	8
test	9
EOF
${prog} -c 'COPY pgoblin_test_table TO stdin' | diff /dev/fd/5 /dev/fd/0

echo 'diff with SELECT BINARY'
6<< "EOF"
begin 644 test
M=&5S=`````!T97-T`````71E<W0````"=&5S=`````-T97-T````!'1E<W0`
C```%=&5S=`````9T97-T````!W1E<W0````(=&5S=`````D`
`
end
EOF
${prog} -S 'SELECT * FROM pgoblin_test_table' | uuencode test | diff /dev/fd/6 /dev/fd/0

echo 'diff with SELECT BINARY'
7<< "EOF"
begin 644 test
M=&5S=`````!T97-T`````71E<W0````"=&5S=`````-T97-T````!'1E<W0`
C```%=&5S=`````9T97-T````!W1E<W0````(=&5S=`````D`
`
end
EOF
${prog} << EOF | uuencode test | diff /dev/fd/7 /dev/fd/0
#binselect 
SELECT * FROM pgoblin_test_table
EOF

echo 'Program'
8<< EOF
#!${prog}
#copyin 
COPY pgoblin_test_table FROM stdin
#strselect 
SELECT t, ' ', i, i * i, '
' FROM pgoblin_test_table
#select 
SELECT t, ' ', i, i * i, '
' FROM pgoblin_test_table
#copyout 
COPY pgoblin_test_table TO stdout
#perform 
DROP TABLE pgoblin_test_table
EOF
9<< EOF
test 00
test 11
test 24
test 39
test 416
test 525
test 636
test 749
test 864
test 981
tset 10100
test 00
test 11
test 24
test 39
test 416
test 525
test 636
test 749
test 864
test 981
tset 10100
test	0
test	1
test	2
test	3
test	4
test	5
test	6
test	7
test	8
test	9
tset	10
EOF
${prog} /dev/fd/8 << EOF | diff /dev/fd/9 /dev/fd/0
tset	10
EOF

cat << EOF > ${TF}
#!${prog}
#perform
CREATE TEMP TABLE p (n int, e text, c text)
;
CREATE TEMP TABLE a (n int, c text)
;
CREATE TEMP TABLE e (n int, e text, c text)
;
CREATE TEMP TABLE g (n int, e text, c text)
#getpost
#copyin
COPY p FROM stdin
#getarg
#copyin
COPY a FROM stdin
#getenv
#copyin
COPY e FROM stdin
#getget
#copyin
COPY g FROM stdin
#echo
  :

#select
SELECT COALESCE(text(n), '?') || ': ' || COALESCE(e, '?') || '=' || COALESCE(c, '?') || '
' FROM e
 WHERE e IN('USER', 'EDITOR', 'SHELL', 'QUERY_STRING')
#select
SELECT COALESCE(text(n), '?') || '=' || COALESCE(c, '?') || '
' FROM a
#select
SELECT COALESCE(text(n), '?') || ': ' || COALESCE(e, '?') || '=' || COALESCE(c, '?') || '
' FROM g
#select
SELECT COALESCE(text(n), '?') || '! ' || COALESCE(e, '?') || '=' || COALESCE(c, '?') || '
' FROM p
#perform
DROP TABLE p;
DROP TABLE a;
DROP TABLE e;
DROP TABLE g;
EOF

chmod a+x ${TF}
export QUERY_STRING='dkfj=dfivj&kdjvn=&=kjf&dsfjhn'
echo 'ier=iig&dfg' | ${TF}
